/*============================================================================
  File:     IndexMissingStats.sql

  Summary:  Be careful... don't just add all of the "missing" indexes! 
			But, this should give you a comprehensive look at what 
			indexes are being suggested by queries that have run. In addition
			to this you might want to check out the DMVPlanCache script to
			see which of the most expensive queries have run - then start
			to analyze those and combine the recommendations (possibly from
			using DTA) to see if the expensive and most-frequently executed 
			queries are actually needing these missing/recommended indexes.
           
  Date:     February 2011

  SQL Server Version: 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

SELECT g.*
FROM sys.dm_db_missing_index_groups AS g
go

SELECT s.*
FROM sys.dm_db_missing_index_group_stats AS s
go

SELECT d.*
FROM sys.dm_db_missing_index_details AS d
go

SELECT d.*
		, s.avg_total_user_cost
		, s.avg_user_impact
		, s.last_user_seek
		, s.unique_compiles
		, s.*
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
ORDER BY s.avg_user_impact DESC
go

-- Be sure to review Bart Duncan's blog:
--  http://blogs.msdn.com/bartd/archive/2007/07/19/are-you-using-sql-s-missing-index-dmvs.aspx
SELECT 
  migs.avg_total_user_cost * (migs.avg_user_impact / 100.0) * (migs.user_seeks + migs.user_scans) AS improvement_measure, 
  'CREATE INDEX [missing_index_' + CONVERT (varchar, mig.index_group_handle) + '_' + CONVERT (varchar, mid.index_handle) 
  + '_' + LEFT (PARSENAME(mid.statement, 1), 32) + ']'
  + ' ON ' + mid.statement 
  + ' (' + ISNULL (mid.equality_columns,'') 
    + CASE WHEN mid.equality_columns IS NOT NULL AND mid.inequality_columns IS NOT NULL THEN ',' ELSE '' END 
    + ISNULL (mid.inequality_columns, '')
  + ')' 
  + ISNULL (' INCLUDE (' + mid.included_columns + ')', '') AS create_index_statement, 
  migs.*, mid.database_id, mid.[object_id]
FROM sys.dm_db_missing_index_groups mig
    INNER JOIN sys.dm_db_missing_index_group_stats migs 
        ON migs.group_handle = mig.index_group_handle
    INNER JOIN sys.dm_db_missing_index_details mid 
        ON mig.index_handle = mid.index_handle
--the WHERE clause sets the limiting factor for creation. Uncomment the following like for details:
WHERE migs.avg_total_user_cost * (migs.avg_user_impact / 100.0) * (migs.user_seeks + migs.user_scans) > 10
ORDER BY migs.avg_total_user_cost * migs.avg_user_impact * (migs.user_seeks + migs.user_scans) DESC
go

-- Don't forget INDEX CONSOLIDATION!
-- Consider taking out the WHERE and then doing index consolidation over the recommendations
-- there might be a minor modification needed to create an even better index with one of the high
-- priority and low priority indexes being combined!